﻿# coding :utf-8
# 
# winExplore v.5.6 du 22.03.2023.
# Un globalPlugin NVDA pour faciliter la navigation par objets, et lister puis utiliser les éléments d'une fenêtre.
# La navigation en mode scan est inspirée de l'extension ObjNav. Le timer provient des extensions de PaulBert.
import globalPluginHandler
import appModuleHandler
import ui
import scriptHandler
from globalCommands import commands
import api, sys
import textInfos
import speech
import controlTypes
from time import sleep
import wx
import gui
import winUser
from winUser import * 
import NVDAObjects
from keyboardHandler import KeyboardInputGesture
import config

from NVDAObjects.IAccessible import WindowRoot,getNVDAObjectFromPoint, getNVDAObjectFromEvent
import UIAHandler
from UIAHandler import handler, IUIAutomationInvokePattern, IUIAutomationLegacyIAccessiblePattern

import controlTypes
if not hasattr(controlTypes, "Role"):
	setattr(controlTypes, "Role", type('Enum', (), dict(
	[(x.split("ROLE_")[1], getattr(controlTypes, x)) for x in dir(controlTypes) if x.startswith("ROLE_")])))
	setattr(controlTypes, "State", type('Enum', (), dict(
	[(x.split("STATE_")[1], getattr(controlTypes, x)) for x in dir(controlTypes) if x.startswith("STATE_")])))
	setattr(controlTypes, "role", type("role", (), {"_roleLabels": controlTypes.roleLabels})) 
try:	# nvda >= 2022.1
	from UIAHandler.utils import createUIAMultiPropertyCondition
except ImportError:
	from UIAUtils import createUIAMultiPropertyCondition
	
import addonHandler
import core
from tones import beep
import threading
from . import shared
from .shared import *
from .shared import updateAddon
from . import applilist

_curAddon = addonHandler.getCodeAddon()
_scriptCategory = _curAddon.manifest['summary']
clientObject = handler.clientObject
cO=clientObject
CPC=clientObject.CreatePropertyCondition

def CTRFocus() :
	try : REASON_FOCUS=controlTypes.REASON_FOCUS
	except : REASON_FOCUS=controlTypes.OutputReason.FOCUS
	return REASON_FOCUS
def CTRCaret() :
	try : REASON_CARET=controlTypes.REASON_CARET
	except : REASON_CARET=controlTypes.OutputReason.CARET
	return REASON_CARET	
	
class RepeatTask(threading.Thread): 
	def __init__(self): 
		self.noNextTask = True
		super(RepeatTask, self).__init__()
		self._stopevent = threading.Event( ) 
	def task (self):	return
	def run(self): 
		while not self._stopevent.isSet() :
			self._stopevent.wait(self._delay) 
			if self.noNextTask: self.noNextTask = False
			else: self.task()
	def stop(self): 
		self.noNextTask = True
		self._stopevent.set() 

class RepeatBeep(RepeatTask): 
	def __init__(self, delay = 2, beep = (150,50) ):
		self._delay = delay
		self.beep = beep
		super(RepeatBeep,self).__init__()
	def task (self):
		beep(150,50)
	
class ListeObjets(wx.Dialog):
	_timer=None
	
	def __init__(self, parent,ID, title, closeIfInactive): 
		super(ListeObjets, self).__init__(parent, ID,title) 
		self.closeIfInactive = closeIfInactive
		self.destroyed = False
		self.doGui()
		
	def update(self) :
		global elems0
		try:
			elems0 =self.getlisteObjets(100) 
			return elems0
		except:
			return ui.message(u"pas d'élément trouvé")
		
	def doGui(self):
		global elems0
		panel = self
		panelSizer = wx.GridBagSizer (6,4)  
		self.rechercheLabel=wx.StaticText(panel, wx.ID_ANY, label=u"Expression de &filtrage : " )
		panelSizer.Add(self.rechercheLabel,pos=(0,1), span= (1,1), flag=wx.EXPAND|wx.LEFT|wx.TOP,border=15)
		self.zoneRecherche = wx.TextCtrl(panel, wx.ID_ANY)
		self.zoneRecherche.Value ="" 
		panelSizer.Add(self.zoneRecherche, pos=(0,2), span= (1,1), flag=wx.EXPAND|wx.TOP,border=10)
		self.typeLabel = wx.StaticText(panel, wx.ID_ANY, label="&Types : " )
		panelSizer.Add(self.typeLabel, pos=(1,0), span= (1,1), flag=wx.EXPAND|wx.LEFT,border=15)
		#the list boxes
		self.typeObjets = wx.ListBox(panel, wx.ID_ANY, style = wx.LB_SINGLE, size = (260, 400))
		self.typeObjets.SetItems([obj for obj in self.getlisteTypes()]) 
		self.typeObjets.Select(0) 
		panelSizer.Add(self.typeObjets, pos=(2,0), span= (5,1), flag=wx.EXPAND|wx.LEFT|wx.BOTTOM, border=10) 
		self.elemLabel = wx.StaticText(panel, wx.ID_ANY, label=u"&Éléments trouvés: " )
		panelSizer.Add(self.elemLabel, pos=(1,1), span= (1,1), flag=wx.EXPAND|wx.LEFT,border=15)
		self.listeObjets = wx.ListBox(panel, wx.ID_ANY,  style = wx.LB_SINGLE, size = (460, 400))
		try :
			elems0=self.update()
			if elems0 and len(elems0)>0:  
				self.listeObjets.SetItems([obj[0] for obj in elems0]) 
				self.listeObjets.Select(0) 
		except: pass
		panelSizer.Add(self.listeObjets, pos=(2,1), span=(5,2), flag=wx.EXPAND|wx.LEFT|wx.BOTTOM,
			border=10)
		# the buttons
		self.rightClickButton=wx.Button(panel, wx.NewId(), label =("Simple clic &droit"), size = (120, 30))
		self.objetNavigateurButton=wx.Button(panel, wx.NewId(), label =(u"L'élément est \n l'&objet  navigateur"), size = (120, 50))
		self.leftClickButton=wx.Button(panel, wx.NewId(), label =("Simple clic &gauche"), size = (120, 30))
		self.leftClickButton.SetDefault()
		self.cancelButton =wx.Button(panel,wx.ID_CANCEL, size = (120, 30))
		panelSizer.Add(self.rightClickButton,(2,3),flag=wx.ALL, border = 10)
		panelSizer.Add(self.leftClickButton,(3,3),flag=wx.ALL, border = 10)  
		panelSizer.Add(self.objetNavigateurButton,(4,3),flag=wx.ALL, border = 10)  
		panelSizer.Add(self.cancelButton,(6,3),flag = wx.ALL, border= 10)
		panel.SetSizerAndFit(panelSizer)
		# the events
		self.zoneRecherche.Bind(wx.EVT_KEY_DOWN, self.onKeydown0)
		# self.typeObjets.Bind(wx.EVT_LISTBOX,self.onElementTypeChange) #pas si validation par espace
		self.typeObjets.Bind(wx.EVT_KEY_DOWN, self.onKeydown1)
		# self.typeObjets.Bind(wx.EVT_SET_FOCUS,self.onObjectTypeListBoxFocus)  
		self.listeObjets.Bind(wx.EVT_KEY_DOWN, self.onKeydown2)
		self.leftClickButton.Bind(wx.EVT_BUTTON, self.onLeftMouseButton) 
		self.leftClickButton.Bind(wx.EVT_KEY_DOWN, self.onKeydownButton)  	
		self.rightClickButton.Bind(wx.EVT_KEY_DOWN, self.onKeydownButton) 		
		self.rightClickButton.Bind(wx.EVT_BUTTON, self.onRightMouseButton)
		self.objetNavigateurButton.Bind(wx.EVT_BUTTON, self.pourNavigationObjet)
		self.cancelButton.Bind(wx.EVT_BUTTON,self.onCancelButton)
		self.cancelButton.Bind(wx.EVT_KEY_DOWN, self.onKeydownButton)
		self.Bind(wx.EVT_ACTIVATE, self.onActivate)
		self.Bind(wx.EVT_CLOSE, self.onClose)
		self.typeObjets.SetFocus()
		self.sayNumberOfElements()

	def onObjectTypeListBoxFocus(self, evt):  
		self.sayNumberOfElements()
		evt.Skip()
	
	def sayNumberOfElements(self):
		def callback (count):
			self._timer=None
			if count>0:
				msg = _(u"%s éléments") %str(count) if count > 1 else (u"Un élément")
				ui.message(msg)
			else: 
				ui.message(u"aucun élément")
		if self._timer is not None:
			self._timer.Stop()
		self._timer = wx.CallLater(200,callback, self.listeObjets.Count) 
		
	def getPositionXY (self):
		global elems1 
		itemSelected=self.listeObjets.GetSelection() 
		try :
			locat=elems1 [itemSelected][1] 
		except:
			locat=self.getlisteObjets(100) [itemSelected][1] 
		xyItem=(int(locat[0])+int(locat[2]/2),int(locat[1])+int(locat[3]/2))
		return xyItem  

	def onClose(self, evt):
		self.Destroy()
		self.destroyed = True  
		evt.Skip()
		
	def onKeydownButton(self, evt):
		keyCode= evt.GetKeyCode()
		if keyCode == wx.WXK_ESCAPE:
			self.Close()
			return
		evt.Skip()
	
	def onKeydown0(self, evt): # recherche
		keyCode= evt.GetKeyCode() 
		if  keyCode ==wx.WXK_UP: 
			if  self.zoneRecherche.HasFocus() :
				core.callLater(200,SendKey ,"Shift+Tab")
				return	
		if keyCode ==wx.WXK_DOWN:  
			if  self.zoneRecherche.HasFocus() :
				core.callLater(200,SendKey ,"Tab")
				return 
		if keyCode in  [wx.WXK_F12,wx.WXK_PAGEUP,wx.WXK_PAGEDOWN ] : 
			if  self.zoneRecherche.HasFocus() :
				filtre= self.zoneRecherche.GetValue()
				if filtre =="" : 
					if keyCode ==wx.WXK_F12 : beep (330, 66) 
					else : ui.message (u"Pas d'expression entrée")
					self.listeObjets.Clear()
					self.listeObjets.SetItems([obj[0] for obj in self.getlisteObjets(100) ]) 
					self.listeObjets.Select(0)
					self.listeObjets.SetFocus() 
					return 
				ui.message (filtre)	
				self.listeObjets.Clear()
				self.listeObjets.SetItems([obj[0] for obj in self.getlisteObjets(100) ]) 
				self.sayNumberOfElements()	
				self.listeObjets.Select(0)
				self.listeObjets.SetFocus() 
				return 
		evt.Skip()		
		
	def onKeydown1(self, evt):  # types d'objets
		keyCode= evt.GetKeyCode()
		if  keyCode ==wx.WXK_SPACE :
			if  self.typeObjets.HasFocus(): 
				global elems1
				index0=self.typeObjets.GetSelection() 
				oldSpeechMode = getSpeechMode()
				setSpeechMode_off()
				speech.cancelSpeech()
				sleep(0.2)
				elems1=self.getlisteObjets(index0 )
				self.listeObjets.SetItems([obj[0] for obj in elems1] )
				api.processPendingEvents()
				sleep(0.2)
				setSpeechMode(oldSpeechMode)			
				self.sayNumberOfElements() # pas de focus dans liste objets avec space
				return 		
		if  keyCode ==wx.WXK_RIGHT :
			if  self.typeObjets.HasFocus(): 
				index=self.listeObjets.GetSelection()
				if index >0 :
					oldSpeechMode = getSpeechMode()
					setSpeechMode_off()
					speech.cancelSpeech()
					sleep(0.2)
					self.listeObjets.Select(index) 
					self.listeObjets.SetFocus() 
					api.processPendingEvents()
					sleep(0.2)
					setSpeechMode(oldSpeechMode)			
					self.listeObjets.Select(index) 
				else :
					self.listeObjets.Select(0) 
					self.listeObjets.SetFocus() 
				return 
		if keyCode ==wx.WXK_LEFT:   
			core.callLater(200,SendKey ,"Shift+Tab")			
			return 		
		if keyCode in  [wx.WXK_F12,wx.WXK_PAGEUP,wx.WXK_PAGEDOWN ]:   
			core.callLater(200,SendKey ,"Alt+f")	
			return 		
		evt.Skip()
		
	def onKeydown2(self, evt): # éléments trouvés 
		keyCode= evt.GetKeyCode()
		if  keyCode ==wx.WXK_RIGHT :
			if  self.typeObjets.HasFocus() or self.listeObjets.HasFocus(): 
				core.callLater(200,SendKey ,"Tab")
				return		
		if keyCode ==wx.WXK_LEFT:   
			core.callLater(200,SendKey ,"Shift+Tab")
			return 
		if keyCode in  [wx.WXK_F12,wx.WXK_PAGEUP,wx.WXK_PAGEDOWN ]:   
			core.callLater(200,SendKey ,"Alt+f")	
			return 				
		evt.Skip()
		
	def onCancelButton(self, evt):
		self.Close()
		evt.Skip()
		
	def onActivate(self, evt):
		if not self.destroyed and self.closeIfInactive and not evt.GetActive():  
			self.Close()
		evt.Skip()

	def msgObjHorsEcran(self) :
		msg=u"Cet élément est hors écran"
		core.callLater(300,ui.message,msg)
		
	def onLeftMouseButton(self,event):
		(x,y)=self.getPositionXY()
		if (x,y)==(0,0) :
			self.msgObjHorsEcran()
			return
		self.Close()
		winUser.setCursorPos(x,y)
		mouse_event(MOUSEEVENTF_LEFTDOWN,0,0,None,None)
		sleep(0.1)
		mouse_event(MOUSEEVENTF_LEFTUP,0,0,None,None)
			
	def onRightMouseButton(self,event):
		(x,y)=self.getPositionXY()
		if (x,y)==(0,0) :
			self.msgObjHorsEcran()
			return		
		self.Close()
		winUser.setCursorPos(x,y)
		mouse_event(MOUSEEVENTF_RIGHTDOWN,0,0,None,None)
		sleep(0.01)
		mouse_event(MOUSEEVENTF_RIGHTUP,0,0,None,None)	
			
	def pourNavigationObjet(self,event):
		(x0,y0)=self.getPositionXY()
		self.Close()
		oldSpeechMode = getSpeechMode()
		setSpeechMode_off()			
		speech.cancelSpeech()
		sleep(0.2)
		api.processPendingEvents()
		def callback():
			obj=api.getDesktopObject().objectFromPoint(x0,y0)
			api.setNavigatorObject(obj)	
			api.moveMouseToNVDAObject(obj)
			speech.speakObject(obj)
		setSpeechMode(oldSpeechMode)
		att=wx.CallLater(600,callback)

	def getlisteTypes(self):
		name=""
		l=[] 
		max=len(listeTypes)
		for i in range (0,max) :
			name =listeTypes[i]
			l.append((name)) 
		return l 
		
	def getlisteObjets(self,iNb): 
		if RepeatBeep() :
			RepeatBeep().stop()
		sleep(0.4)
		th=None
		th = RepeatBeep()  
		th.start()
		# beep(122,66)
		locat=None 
		name=""
		l=[] 
		# oCondition0=CPC(30173,80051) # titres pour edge
		oCondition0=cO.CreateOrCondition(CPC(30003 ,50000),CPC(30003 ,50031))  # button splitbutton
		oCondition5=cO.CreateOrCondition(CPC(30003 ,50008),CPC(30003 ,50007))
		oCondition6=cO.CreateOrCondition(CPC(30003 ,50015),CPC(30003 ,50003))
		oCondition8=cO.CreateOrCondition(CPC(30003 ,50002),CPC(30003 ,50013))		
		oCondition9=createUIAMultiPropertyCondition ({UIAHandler.UIA_ControlTypePropertyId:[50000,50031,50008,50007,50019,50024,50004,50015,50003,50002,50013,50011,50005]}) 
		conditions=((oCondition0),(CPC(30003 ,50019)),(CPC(30003 ,50024)),(CPC(30003 ,50005)),(CPC(30003 ,50004)),(oCondition5),(oCondition6),(CPC(30003 ,50011)),(oCondition8),(oCondition9))
		n=0
		if iNb<10 :n=iNb
		else :
			try: n=self.typeObjets.GetSelection()
			except : n=0 ; pass
		n=self.typeObjets.GetSelection()
		oCondition =conditions[n]
		try :
			hd =oDeb.windowHandle		
			elements = clientObject.ElementFromHandleBuildCache(hd,UIAHandler.handler.baseCacheRequest).FindAllBuildCache(4,oCondition,UIAHandler.handler.baseCacheRequest)		
			# elements = clientObject.ElementFromHandle(hd).FindAll(4,oCondition)
			th.stop()
			del th			
			
		except:
			print (u"Erreur dans la liste des éléments")
			th.stop()
			del th
			ui.message(u"pas d'élément ou éléments non trouvés")
			return l
		try : 
			if not elements.Length :	
				ui.message(u"pas d'élément trouvé")
				return l
		except : 
			if RepeatBeep() :
				RepeatBeep().stop()
			return l	
		try : 	
			if elements.Length :		
				filtre=self.zoneRecherche.GetValue().lower()
				for i in range (0, elements.Length):  
					name=""
					if elements.GetElement(i).CurrentName.lower() in ("ligne vers le haut","ligne vers le bas",u"page précédente",u"page suivante",u"page vers la gauche",u"page vers la droite","colonne vers la gauche","colonne vers la droite","> ","< ", " "):
						continue
					iRole=elements.GetElement(i).GetCurrentPropertyValue(30095)
					if iRole in [10,20] :
						continue
					if elements.GetElement(i).CurrentName!="":
						name=elements.GetElement(i).CurrentName
					elif elements.GetElement(i).GetCurrentPropertyValue(30045): #value
						name=elements.GetElement(i).GetCurrentPropertyValue(30045)
					if filtre and  (not filtre in name.lower()) : continue  
					if name=="" : continue 
					iRole = elements.GetElement(i).GetCurrentPropertyValue(30095)
					if n==9 or (n<9 and iRole in (33,43,44,45,49,58,62,64)) :
						name1=nomRolesMSAA[iRole]
						name = name+" - "+name1
					if iRole in (44,45):
						try :						
							if  elements.GetElement(i).GetCurrentPropertyValue(30086)==1 :
								name=name+u" cochée"
							elif  elements.GetElement(i).GetCurrentPropertyValue(30086)==0 :
								name=name+u" non cochée"
							elif  elements.GetElement(i).GetCurrentPropertyValue(30079)==1 :
								name=name+u" coché"
							elif  elements.GetElement(i).GetCurrentPropertyValue(30079)==0 :
								name=name+u" non coché"									
						except :
							pass
					if iRole in (43,49,56,57,58,62,64):
						try :
							val=hex(elements.GetElement(i).GetCurrentPropertyValue(30096)).split("x")[1]
							val1=int(val[-1:])
							if val1==1 : name=name+u" inactif"
							if val1==8 : name=name+u" enfoncé"
						except:
							pass
					if name!="" :
						locat =elements.GetElement(i).GetCurrentPropertyValue(30001)
						l.append((name,locat)) 
		except : 
			pass
		if RepeatBeep() :
			RepeatBeep().stop()
		return l 	
		
	@classmethod	
	def run(cls): 
		bdd = cls(None,wx.ID_ANY,(u"Liste des éléments dans la fenêtre"), True)
		bdd.CenterOnScreen()
		bdd.Show(True)
		PutWindowOnForeground(bdd.GetHandle(), 4, 0.1) 
	
class ListeRubObjets(wx.Dialog):  
	_timer=None
	indexGroupes=0
	nbGroupes=0
	
	global listeOnglets, indexOnglet,  oRub, avecDescription
	avecDescription =False
	
	def __init__(self, parent,ID, title, closeIfInactive): 
		super(ListeRubObjets, self).__init__(parent, ID,title) 
		self.closeIfInactive = closeIfInactive
		self.destroyed = False 
		self.getlisteRubGroupes()
		self.getlisteRubObjets()  
		self.objectTypeHasChanged = True
		self.doGui()

	def doGui(self):
		panel = self
		panelSizer = wx.GridBagSizer (4,4) 
		self.taggedObjectsCB = wx.CheckBox(panel, wx.ID_ANY, label =u"&Ajouter les descriptions aux noms, quand elles existent")
		self.taggedObjectsCB.SetValue(False)  
		panelSizer.Add(self.taggedObjectsCB, pos=(0,0), span= (1,2), flag=wx.EXPAND|wx.ALL,border=15)
		self.typeLabel = wx.StaticText(panel, wx.ID_ANY, label="&Onglets, Entrée pour en changer :" , size = (190,10)) 		
		panelSizer.Add(self.typeLabel, pos=(1,0), span= (1,1), flag=wx.EXPAND|wx.LEFT,border=15) 
		self.lbOnglets = wx.ListBox(panel, wx.ID_ANY, style = wx.LB_SINGLE, size = (190, 250))
		try : 
			if len(listeOnglets)>0: 
				self.lbOnglets.SetItems([obj[0] for obj in listeOnglets])  
				self.lbOnglets.Select(indexOnglet)
		except : pass
		panelSizer.Add(self.lbOnglets, pos=(2,0), span= (1,1), flag=wx.EXPAND|wx.LEFT,border=15)		
		self.typeLabel = wx.StaticText(panel, wx.ID_ANY, label="Grou&pes dans l'onglet actuel : " , size = (190,10))
		panelSizer.Add(self.typeLabel, pos=(1,1), span= (1,1), flag=wx.EXPAND|wx.LEFT,border=7) 
		self.lbGroupes = wx.ListBox(panel, wx.ID_ANY, style = wx.LB_SINGLE, size = (190, 250)) 
		self.lbGroupes.SetItems([obj for obj in self.getlisteRubGroupes()])  
		try :
			if self.indexGroupes : self.lbGroupes.Select(self.indexGroupes)
			else : self.lbGroupes.Select(0)
		except : 
			print("pb lbgroupes")   #<<<<<<<
			pass
		panelSizer.Add(self.lbGroupes, pos=(2,1), span= (1,1), flag=wx.EXPAND|wx.LEFT|wx.RIGHT,
			border=7)  
		self.elemLabel = wx.StaticText(panel, wx.ID_ANY, label=u"&Éléments dans le groupe :" )
		panelSizer.Add(self.elemLabel, pos=(1,2), span= (1,1), flag=wx.EXPAND,border=15)  
		self.lbObjets = wx.ListBox(panel, wx.ID_ANY,  style = wx.LB_SINGLE,  size = (480, 250)) 
		try :
			if len(self.getlisteRubGroupes())>0: 
				self.lbObjets.SetItems([obj[0] for obj in self.getlisteRubObjets()])  
		except : pass
		panelSizer.Add(self.lbObjets, pos=(2,2), span=(1,2), flag=wx.EXPAND|wx.RIGHT,
			border=15)  
		# the buttons
		self.rightClickButton=wx.Button(panel, wx.NewId(), label =("Simple clic &droit"), size = (120, 30))
		self.leftClickButton=wx.Button(panel, wx.NewId(), label =("Simple clic &gauche"), size = (120, 30))
		self.objetNavigateurButton=wx.Button(panel, wx.NewId(), label =(u"L'élément est l'&objet  navigateur"), size = (200, 30))
		self.cancelButton =wx.Button(panel,wx.ID_CANCEL, size = (120, 30))
		self.leftClickButton.SetDefault()
		panelSizer.Add(self.rightClickButton,(3,0),flag=wx.ALL, border = 15)
		panelSizer.Add(self.leftClickButton,(3,1),flag=wx.TOP|wx.BOTTOM|wx.LEFT, border = 15)
		panelSizer.Add(self.objetNavigateurButton,(3,2),flag=wx.ALL, border = 15)  
		panelSizer.Add(self.cancelButton,(3,3),flag = wx.TOP|wx.BOTTOM|wx.LEFT, border = 15)
		#panelSizer.Add((-1,40))
		panel.SetSizerAndFit(panelSizer)
		# the events
		self.taggedObjectsCB.Bind(wx.EVT_CHECKBOX,self.onCaseChange)  
		self.lbOnglets.Bind(wx.EVT_KEY_DOWN, self.onKeydown0)
		self.lbGroupes.Bind(wx.EVT_LISTBOX,self.onElementTypeChange) 
		# self.lbGroupes.Bind(wx.EVT_SET_FOCUS,self.onObjectTypeListBoxFocus) 
		self.lbGroupes.Bind(wx.EVT_KEY_DOWN, self.onKeydown1)		
		self.lbObjets.Bind(wx.EVT_KEY_DOWN, self.onKeydown2)
		self.leftClickButton.Bind(wx.EVT_BUTTON, self.onLeftMouseButton) 
		self.leftClickButton.Bind(wx.EVT_KEY_DOWN, self.onKeydownButton)  	
		self.rightClickButton.Bind(wx.EVT_KEY_DOWN, self.onKeydownButton) 
		self.objetNavigateurButton.Bind(wx.EVT_BUTTON, self.pourNavigationObjet)		
		self.rightClickButton.Bind(wx.EVT_BUTTON, self.onRightMouseButton)
		self.cancelButton.Bind(wx.EVT_BUTTON,self.onCancelButton)
		self.cancelButton.Bind(wx.EVT_KEY_DOWN, self.onKeydownButton)
		self.Bind(wx.EVT_ACTIVATE, self.onActivate)
		self.Bind(wx.EVT_CLOSE, self.onClose)
		self.lbGroupes.SetFocus()
		self.sayNumberOfElements()
	
	def onCaseChange(self, evt): 
		global avecDescription  
		avecDescription =self.taggedObjectsCB.GetValue() 
		index=(self.getIndexObjets() if self.getIndexObjets() else 0)
		if avecDescription  : ui.message ("Avec description")
		else : ui.message ("Sans description")
		self.lbObjets.Clear()
		self.lbObjets.SetItems([obj[0] for obj in self.getlisteRubObjets()  ]) 
		self.lbObjets.SetFocus() 
		self.lbObjets.Select(index) 
		self.objectTypeHasChanged = True
		return
		evt.Skip()

	def onElementTypeChange(self, evt): 
		if self._timer :
			self._timer.Stop()
		self.indexGroupes=evt.GetInt()  # indexObjets
		if self.indexGroupes>=0:
			self.lbObjets.Clear()
			self.lbObjets.SetItems([obj[0] for obj in self.getlisteRubObjets() ]) 		
			self.sayNumberOfElements()
			self.objectTypeHasChanged = True
		return
		evt.Skip()  	
		
	def onObjectTypeListBoxFocus(self, evt):  
		self.sayNumberOfElements()
		return
		evt.Skip() 		
	
	def sayNumberOfElements(self):
		def callback (count):
			self._timer=None
			if not self.lbGroupes.HasFocus(): return
			if count:
				msg = _(u"%s éléments") %str(count) if count > 1 else (u"Un élément")
				ui.message(msg)
			else:
				ui.message(u"aucun élément")
		if self._timer is not None:
			self._timer.Stop()
		self._timer = wx.CallLater(200,callback, self.lbObjets.Count) 
		
	def getIndexObjets(self):
		nb= int(self.lbObjets.GetSelection())
		if nb<0 : nb=0
		return nb
	
	def getPositionXY (self):
		if  self.lbOnglets.HasFocus() :
			itemSelected=self.lbOnglets.GetSelection() 
			locat=listeOnglets [itemSelected][1]
		else :
			itemSelected=self.lbObjets.GetSelection() 
			locat=self.getlisteRubObjets() [itemSelected][1] 
		xyItem=(int(locat[0])+int(locat[2]/2),int(locat[1])+int(locat[3]/2))
		return xyItem  
		
	def onClose(self, evt):
		self.Destroy()
		self.destroyed = True 
		evt.Skip()

	def getlisteRubOnglets(self) :
		global  listeOnglets,  indexOnglet , oRub 	#oRub = Ruban inferieur
		try: 
			o1 =oRub.parent
			for i in range(0, o1.childCount):
				if "onglets du ruban" in o1.children[i].name.lower() :
					break
			o=o1.children[i]
			if o.firstChild.name=="" :
				o=o.firstChild
			n=len(o.children)
			ui.message(u"Onglets du ruban : Fichier, ")
			for i in range(n):
				# if STATE_CHECKED in obj.states:
				# if  not STATE_INVISIBLE in o.children[i].states:  
				if  not controlTypes.State.INVISIBLE in o.children[i].states:  
					name =o.children[i].name		
					ui.message(name)
		except : return ui.message (u"Erreur d'exécution très sommaire.") 	
		
	def getlisteRubGroupes(self) :
		global  listeOnglets,  indexOnglet , oRub 	#oRub = Ruban inferieur
		locat=None 
		name=""
		l=[] 
		listeOnglets=[]
		indexOnglet=0
		nomOnglet=""
		o1=oRub
		if o1.firstChild.name=="" :
			o1=o1.firstChild
		nomOnglet=o1.firstChild.name
		self.SetTitle(u'Éléments dans l\'onglet de ruban "'+nomOnglet+'".')
		# liste des onglets
		o1=oRub.parent 
		for i in range(0, o1.childCount):
			if "onglets du ruban" in o1.children[i].name.lower() :
				break
		o=o1.children[i]
		if o.firstChild.name=="" :
			o=o.firstChild
		n=len(o.children)
		j=0
		for i in range(0,n):
			if  not controlTypes.State.INVISIBLE in o.children[i].states:   
				name =o.children[i].name
				if name==nomOnglet : 
					indexOnglet=j
				j+=1
				locat=o.children[i].location
				listeOnglets.append((name,locat)) 		
		# liste des groupes
		o=oRub.firstChild
		if o.name=="" :
			o=o.firstChild		
		n=len(o.children)
		nbGroupes=n
		for i in range(0,n): 
			if   o.children[i].role in (controlTypes.Role.TOOLBAR,controlTypes.Role.GROUPING)  :  
				if  controlTypes.State.INVISIBLE not in o.children[i].states: 
					name =o.children[i].name  if o.children[i].name!="" else "Sans nom"
					l.append((name))	
		return l
			
	def getlisteRubObjets(self):  	
		global oRub,avecDescription, indexObjets 
		locat=None 
		name=""
		elem=None
		l=[] 
		try: 
			elements00=cO.ElementFromHandle(oRub.windowHandle).FindFirst(4,CPC(30095,38)) #explorateur 38 page propriet
			elements0=elements00.FindAll(2,CPC(30003,50021))  #toolbar
			elem=elements0.GetElement(self.indexGroupes)
		except :
			try: 
				elements00=cO.ElementFromHandle(oRub.windowHandle).FindFirst(4,CPC(30003,50025)) #CustomControl 
				elements0=elements00.FindAll(2,CPC(30003,50026))  #groupe
				elem=elements0.GetElement(self.indexGroupes)  #lbGroupes.GetSelection())
			except :				
				try :
					elements00=cO.ElementFromHandle(oRub.windowHandle).FindFirst(4,CPC(30003,50026)) 
					elements0=elements00.FindAll(2,CPC(30003,50026))  # office 2019  groupe pour Rub
					elem=elements0.GetElement(self.indexGroupes)  #self.lbObjets.GetSelection())
				except :
					print (u"problème dans getlisteObjets")
					return
		oCondition=cO.CreateTrueCondition()
		elements=elem.FindAll(4,oCondition) 
		if elements.Length : 
			for i in range (0,elements.Length):   
				name,name0="",""
				if elements.GetElement(i).CurrentName.lower() in ("ligne précédente","ligne suivante",u"page précédente",u"page suivante",u"page vers la gauche",u"page vers la droite","colonne vers la gauche","colonne vers la droite"):
					continue
				iRole=elements.GetElement(i).GetCurrentPropertyValue(30095)  
				if iRole in [10,20,40] :  # MSAA client, groupe, image
					continue
				if elements.GetElement(i).CurrentName!="":
					name0=elements.GetElement(i).CurrentName
					name=name0
				else : name = "Sans nom" 
				name1=nomRolesMSAA [iRole]
				name = name+" - "+name1	
				if iRole in (44,45):
					try :
						if  elements.GetElement(i).GetCurrentPropertyValue(30086)==1 :
							name=name+u" cochée"
						elif  elements.GetElement(i).GetCurrentPropertyValue(30086)==0 :
							name=name+u" non cochée"
						elif  elements.GetElement(i).GetCurrentPropertyValue(30079)==1 :
							name=name+u" coché"
						elif  elements.GetElement(i).GetCurrentPropertyValue(30079)==0 :
							name=name+u" non coché"		
					except :
						pass
				if iRole in (43,49,56,57,58,62,64):
					try :
						val=hex(elements.GetElement(i).GetCurrentPropertyValue(30096)).split("x")[1]
						val1=int(val[-1:])
						if val1==1 : name=name+u" inactif"
						if val1==8 : name=name+u" enfoncé"
					except:
						pass
				if avecDescription and elements.GetElement(i).GetCurrentPropertyValue(30094): 
				# if self.taggedObjectsCB.GetValue()  and elements.GetElement(i).GetCurrentPropertyValue(30094): 
					if name0 in elements.GetElement(i).GetCurrentPropertyValue(30094) :
						name3=""
					else :
						name3=elements.GetElement(i).GetCurrentPropertyValue(30094)
					name=name+" - "+name3
				locat =elements.GetElement(i).GetCurrentPropertyValue(30001)
				l.append((name,locat))	
		return l								
			
	def updateButtons(self,enable=True): 
		if enable:
			self.leftClickButton.Enable()
			self.rightClickButton.Enable()
		else:
			self.leftClickButton.Disable()
			self.rightClickButton.Disable()
		
	def onKeydownButton(self, evt):
		keyCode= evt.GetKeyCode()
		if keyCode == wx.WXK_ESCAPE:
			self.Close()
			return
		evt.Skip()
	
	def onKeydown0(self, evt): # pour liste d'onglets
		keyCode= evt.GetKeyCode()
		if  keyCode ==wx.WXK_RIGHT :
			if  self.lbOnglets.HasFocus(): 
				index1=self.lbGroupes.GetSelection()
				if index1 >0 :
					oldSpeechMode = getSpeechMode()
					setSpeechMode_off()
					speech.cancelSpeech()
					sleep(0.2)
					self.lbGroupes.Select(index1) 
					self.lbGroupes.SetFocus() 
					api.processPendingEvents()
					sleep(0.2)
					setSpeechMode(oldSpeechMode)				
					self.lbGroupes.SetSelection(index1) 
				else :
					self.lbGroupes.SetFocus() 
				return 
		if keyCode ==wx.WXK_LEFT:   
			core.callLater(200,SendKey ,"Shift+Tab")
			return 
		evt.Skip()	
		
	def onKeydown1(self, evt):  # pour liste de groupes
		keyCode= evt.GetKeyCode()
		if  keyCode ==wx.WXK_RIGHT :
			if  self.lbGroupes.HasFocus(): 
				index0=self.getIndexObjets()
				if index0 >0 :
					oldSpeechMode = getSpeechMode()
					setSpeechMode_off()
					speech.cancelSpeech()
					sleep(0.2)
					self.lbObjets.Select(index0) 
					self.lbObjets.SetFocus() 
					api.processPendingEvents()
					sleep(0.2)
					setSpeechMode(oldSpeechMode)			
				else :
					if len(self.getlisteRubObjets()) : self.lbObjets.Select(0) 
					self.lbObjets.SetFocus() 
				return	
		if keyCode ==wx.WXK_LEFT:   	
			core.callLater(200,SendKey ,"Shift+Tab")
			return 
		if keyCode in [wx.WXK_F12,wx.WXK_PAGEUP,wx.WXK_PAGEDOWN]:   	
			core.callLater(200,SendKey ,"Alt+a")
			return 			
		evt.Skip()
		
	def onKeydown2(self, evt):   # pour liste d'objets
		keyCode= evt.GetKeyCode()
		if  keyCode ==wx.WXK_RIGHT :
			if  self.lbObjets.HasFocus() : 
				core.callLater(200,SendKey ,"Tab")
				return		
		if keyCode ==wx.WXK_LEFT:   		
			core.callLater(200,SendKey ,"Shift+Tab")
			return 
		if keyCode in [wx.WXK_F12,wx.WXK_PAGEUP,wx.WXK_PAGEDOWN]:   	
			core.callLater(200,SendKey ,"Alt+a")
			return 				
		evt.Skip()
		
	def onCancelButton(self, evt):
		self.Close()
		evt.Skip()
		
	def onActivate(self, evt):
		if not self.destroyed and self.closeIfInactive and not evt.GetActive():  
			self.Close()
		evt.Skip()
		
	def onLeftMouseButton(self,event):
		HF=False
		if  self.lbOnglets.HasFocus() : HF=True
		if  self.lbGroupes.HasFocus() : return
		(x,y)=self.getPositionXY()
		self.Close()
		winUser.setCursorPos(x,y)
		mouse_event(MOUSEEVENTF_LEFTDOWN,0,0,None,None)
		sleep(0.1)
		mouse_event(MOUSEEVENTF_LEFTUP,0,0,None,None)	
		if HF==True :
			wx.CallLater(400,ListeRubObjets.run())
	
	def onRightMouseButton(self,event):
		if  self.lbOnglets.HasFocus() : return
		if  self.lbGroupes.HasFocus() : return
		(x,y)=self.getPositionXY()
		self.Close()
		winUser.setCursorPos(x,y)
		mouse_event(MOUSEEVENTF_RIGHTDOWN,0,0,None,None)
		sleep(0.01)
		mouse_event(MOUSEEVENTF_RIGHTUP,0,0,None,None)		
		mouse_event(MOUSEEVENTF_RIGHTUP,0,0,None,None)	

	def pourNavigationObjet(self,event):
		(x0,y0)=self.getPositionXY()
		self.Close()
		oldSpeechMode = getSpeechMode()
		setSpeechMode_off()
		speech.cancelSpeech()
		sleep(0.2)
		api.processPendingEvents()
		def callback():
			obj=api.getDesktopObject().objectFromPoint(x0,y0)
			api.setNavigatorObject(obj)	
			api.moveMouseToNVDAObject(obj)
			speech.speakObject(obj)
		setSpeechMode(oldSpeechMode)
		att=wx.CallLater(600,callback)		
		
	def run(cls): 
		bdd = cls(None,wx.ID_ANY,(u"Liste des éléments dans l'onglet de ruban séléctionné"), True)  
		bdd.CenterOnScreen()
		bdd.Show(True)
		PutWindowOnForeground(bdd.GetHandle(), 5, 0.1)  #4
	run = classmethod(run)
	
class GlobalPlugin(globalPluginHandler.GlobalPlugin):
	objArrowMode = 0
	timer = None
	timerStartedAt = 0
	def __init__(self, *args, **kwargs):
		super (GlobalPlugin, self).__init__(*args, **kwargs)
		wx.CallLater(10000, updateAddon.checkUpdate, True,False)  
		
	def event_gainFocus(self, obj, nextHandler):
		try:
			if api.getFocusObject().windowClassName== "Windows.UI.Input.InputSite.WindowClass":
				self.bindTableNavGestures()
			else:
				self.clearGestureBindings()
				self.bindGestures(self.__gestures)
		except WindowsError:
			pass
		nextHandler()	

	def bindTableNavGestures(self):
		self.bindGesture("kb:alt+shift+UpArrow", "litIconesSelectedDrag")
		self.bindGesture("kb:alt+shift+RightArrow", "deplaceDroite")
		self.bindGesture("kb:alt+shift+LeftArrow", "deplaceGauche")
		
	def script_changeMode(self, gesture):
		if self.objArrowMode == 0:
			self.objArrowMode = 1
			ui.message(u"mode séquentiel")
		else:
			self.objArrowMode = 0
			ui.message(u"mode groupé")
		if self.objArrowMode ==1: 
			self.bindGesture("kb:rightarrow", "rightArrow")
			self.bindGesture("kb:leftarrow", "leftArrow")
			self.bindGesture("kb:downarrow", "downArrow")
			self.bindGesture("kb:uparrow", "upArrow")
			self.bindGesture("kb:pageDown", "pageDown")
			self.bindGesture("kb:pageUp", "pageUp")
			self.bindGesture("kb:home", "firstObject")
			self.bindGesture("kb:enter", "activationObjetNavigateur")
			for e in ("kb:*","kb:$","kb:space") :
				self.bindGesture(e, "retourModeNormal")
		else:
			self.clearGestureBindings()
			self.bindGestures(self.__gestures)	
	script_changeMode.__doc__ = _(u'Permuter entre le mode séquentiel et le mode global pour les commandes de navigation')
	script_changeMode.category = _scriptCategory		
	
	def script_rightArrow(self, gesture):
		commands.script_navigatorObject_next(gesture)

	def script_leftArrow(self, gesture):
		commands.script_navigatorObject_previous(gesture)

	def script_downArrow(self, gesture):
		commands.script_navigatorObject_firstChild(gesture)
			
	def script_upArrow(self, gesture):
		commands.script_navigatorObject_parent(gesture)
			
	def script_firstObject(self, gesture):
		curObject=api.getForegroundObject() if api.getForegroundObject().role==1 else api.getForegroundObject().simpleParent
		api.setNavigatorObject(curObject)
		REASON_FOCUS=CTRFocus()
		speech.speakObject(curObject,reason=REASON_FOCUS)
		
	def script_pageDown(self, gesture):
		# Navigate to next line if possible.
		info=api.getReviewPosition().copy()
		info.expand(textInfos.UNIT_LINE)
		info.collapse()
		res=info.move(textInfos.UNIT_LINE,1)
		if res!=0:
			api.setReviewPosition(info)
			info.expand(textInfos.UNIT_LINE)
			REASON_CARET=CTRCaret()
			speech.speakTextInfo(info,unit=textInfos.UNIT_LINE,reason=REASON_CARET)
		else:
			curObject=api.getNavigatorObject()
			newObject=None
			if curObject.simpleFirstChild:
				newObject=curObject.simpleFirstChild
			elif curObject.simpleNext:
				newObject=curObject.simpleNext
			elif curObject.simpleParent:
				parent=curObject.simpleParent
				while parent and not parent.simpleNext:
					parent=parent.simpleParent
				# As long as one is on current foreground object...
				#Stay within the current top-level window.
				if parent and parent.simpleParent != api.getDesktopObject():
					newObject=parent.simpleNext
			if newObject:
				api.setNavigatorObject(newObject)
				speech.speakObject(newObject,reason=controlTypes.OutputReason.FOCUS)
			else:
				ui.reviewMessage(u"Pas d'objet suivant")
		
	def script_pageUp(self, gesture):
		# Move to previous line first so text can be reviewed before resorting to a new object.
		info=api.getReviewPosition().copy()
		info.expand(textInfos.UNIT_LINE)
		info.collapse()
		res=info.move(textInfos.UNIT_LINE,-1)
		if res!=0:
			api.setReviewPosition(info)
			info.expand(textInfos.UNIT_LINE)
			REASON_CARET=CTRCaret()
			speech.speakTextInfo(info,unit=textInfos.UNIT_LINE,reason=REASON_CARET)
		else:
			# Do not move outside of the current window.
			curObject=api.getNavigatorObject()
			newObject=None
			if curObject.parent != api.getDesktopObject():
				newObject=curObject.simplePrevious
				if newObject:
					while newObject.simpleLastChild:
						newObject=newObject.simpleLastChild
				else:
					newObject=curObject.simpleParent
			if newObject:
				api.setNavigatorObject(newObject)
				REASON_FOCUS=CTRFocus()
				speech.speakObject(curObject,reason=REASON_FOCUS)
			else:
				ui.reviewMessage(u"Pas d'objet précédent")		

	def script_activationObjetNavigateur(self, gesture): 
		if not config.conf["reviewCursor"]["simpleReviewMode"]:
			i=0
			oldSpeechMode = getSpeechMode()
			setSpeechMode_off()
			speech.cancelSpeech()
			sleep(0.2)
			api.processPendingEvents()
			commands.script_reviewMode_previous(gesture)
			sleep(0.3)
			if not config.conf["reviewCursor"]["simpleReviewMode"]:
				commands.script_reviewMode_previous(gesture)
				sleep(0.3)
			commands.script_review_activate(gesture)
			sleep(0.3)
			commands.script_reviewMode_next(gesture)
			sleep(0.3)
			if i==2:
				commands.script_reviewMode_next(gesture)
				sleep(0.3)
			setSpeechMode(oldSpeechMode)	
		else:
			commands.script_review_activate(gesture)
			
	def script_retourModeNormal(self, gesture):
		if self.objArrowMode ==1:
			self.clearGestureBindings()
			self.bindGestures(self.__gestures)
			self.objArrowMode =0
			ui.message("mode normal")						
			
	def pourRubList(self):		
		global oRub 
		oRub=objRub()
		if oRub and u"inférieur" in oRub.name: ListeRubObjets.run()
		else : return ui.message (u"Fenêtre sans ruban." )			
				
	def pourWinList(self):			
		if versionWLM(self)>0 :  
			if nombreMessagesWLM (self)  ==1000:  
				ui.message (u"Ce traitement serait trop long. Choisissez un dossier d'arborescence avec moins de 100 messages."  )  # return 
		global oDeb
		oDeb= getRootAppli()  
		ListeObjets.run()
			
	def script_listeElemFenetre(self,gesture) :
		if u"Liste des éléments" in winUser.getWindowText(winUser.getForegroundWindow()):
			return False
		else : 
			self.pourWinList()
	script_listeElemFenetre.__doc__ = _(u'Afficher le dialogue "Liste des éléments" dans la fenêtre.')
	script_listeElemFenetre.category = _scriptCategory		
	
	def script_listeElemRuban(self,gesture) :
		o=api.getFocusObject() 		
		if ( o.windowClassName=="SysListView32" and o.role in (14,15)):
			return ui.message (u"Bureau.")
		if "onglet de ruban" in winUser.getWindowText(winUser.getForegroundWindow()):
			return False
		else :
			global oRub
			oRub=objRub()
			if oRub :self.pourRubList() 
	script_listeElemRuban.__doc__ = _(u'Afficher le dialogue "Éléments" dans l\'onglet actif du ruban s\'il existe.')
	script_listeElemRuban.category = _scriptCategory		
	
	def script_statusBar(self, gesture):	
		try :
			if appModuleHandler.getAppNameFromProcessID(api.getFocusObject().processID,True) =="explorer.exe" : 
				shared.pourBarreExplorer(self)
			elif appModuleHandler.getAppNameFromProcessID(api.getFocusObject().processID,True) in ["firefox.exe" ,"waterfox.exe"]: 			
				shared.pourBarreFirefox(self)		
			else : 
				commands.script_reportStatusLine(gesture)
		except : 
			gesture.send()
			return
	script_statusBar.__doc__ = (u"Lit la ligne d'état. Une double ou une triple frappe la copie dans le presse-papier.")
	script_statusBar.category = _scriptCategory	
	
	def isRibbonRoot(obj):
		if obj.name == "Ribbon" and obj.role == 3:  #roles.PANE
			return True
		return False					
	
	def script_ceTest(self, gesture):	
		ui.message ("Test !") 
		# print (sys.getwindowsversion().major )
		# print (sys.getwindowsversion().build )
		# from datetime import datetime
		# datatt=datetime.now()
		# print(datatt)
		
	script_ceTest.__doc__ = (u"Lit la ligne d'état. Une double ou une triple frappe la copie dans le presse-papier.")
	
	def script_clicCentreEcran(self, gesture):	
		try:
			obj=api.getForegroundObject()
			if u"barre des tâches" in obj.name.lower() : 
				(x,y) = wx.GetDisplaySize()
				winUser.setCursorPos(int(x/2),int(y/2))
				mouse_event(MOUSEEVENTF_LEFTUP,0,0,None,None)
				sleep(0.1)
				mouse_event(MOUSEEVENTF_LEFTDOWN,0,0,None,None)
				return ui.message ("Clic gauche de souris au centre de l'écran.")
			else : 
				beep (222,44)
				return
		except : 
			beep (222,44)
			return
	script_clicCentreEcran.__doc__ = u"Clic gauche de souris au centre de l'écran."
	script_clicCentreEcran.category = _scriptCategory
	
	def script_ditOngletsRuban(self, gesture):	
		o=api.getFocusObject() 		
		if ( o.windowClassName=="SysListView32" and o.role in (14,15)):
			return ui.message (u"Bureau.")
		try :
			if u"dans l'onglet" in api.getForegroundObject().name :
				ListeRubObjets.getlisteRubOnglets(self)
			else : 
				getlisteRubOnglets(self) 
		except : 
			getlisteRubOnglets(self) 	
	script_ditOngletsRuban.__doc__ = u"Vocalise les onglets du ruban"
	script_ditOngletsRuban.category = _scriptCategory	
	
	def script_searchUpdate(self, gesture):	
		twice=(True if (scriptHandler.getLastScriptRepeatCount()>0) else False) 
		wx.CallLater(100,updateAddon.checkUpdate, False,twice)
	script_searchUpdate.__doc__ = u"Recherche une mise à jour de winExplore"
	
	def script_applisBarreTaches(self, gesture):	
		if sys.getwindowsversion().major < 10 :
			wx.CallAfter(applilist.ElementsListDialog.run,True)
		else : 
			applilist.ElementsListDialog.run(False) 
	script_applisBarreTaches.__doc__ = u"Lister et activer les applications ouvertes."
	script_applisBarreTaches.category = _scriptCategory
	
	def script_applisBarreTaches2(self, gesture):	
		applilist.ElementsListDialog.run(True)
	script_applisBarreTaches2.__doc__ = u"Lister et activer les applications de la barre des tâches."
	script_applisBarreTaches2.category = _scriptCategory
	
	def script_deplaceDroite(self,gesture):
		applilist.ElementsListDialog.vocalSelectIcone11(self,2)
	script_deplaceDroite.__doc__ = u"Déplace vers la droite l'icône active dans la barre des tâches."
	script_deplaceDroite.category = _scriptCategory
	
	def script_deplaceGauche(self,gesture):
		applilist.ElementsListDialog.vocalSelectIcone11(self,1)
	script_deplaceGauche.__doc__ = u"Déplace vers la gauche l'icône active dans la barre des tâches."
	script_deplaceGauche.category = _scriptCategory
	
	def script_litIconesSelectedDrag(self,gesture):
		applilist.ElementsListDialog.vocalSelectIcone11(self,0)
	script_litIconesSelectedDrag.__doc__ = u"Lire l'icône active dans la barre des tâches et les deux icônes voisines."
	script_litIconesSelectedDrag.category = _scriptCategory 
	
	def script_listeCommandesExtension(self,gesture):
		applilist.CommandsListDialog.run()
	script_listeCommandesExtension.__doc__ = u"Lister les commandes disponibles."
	script_listeCommandesExtension.category = _scriptCategory	
	
	def script_reportClipboardText(self,gesture):
		try:
			text = api.getClipData()
		except:
			text = None
		if not text or not isinstance(text,str) or text.isspace():
			ui.message("Le presse-papier est vide.")
			return
		if len(text) < 4096:  #1024
			ui.message(text)
		else:
			ui.message((u"Le presse-papier contient une grande quantité de texte : %s caractères") % len(text))
	script_reportClipboardText.__doc__ = _("Reports the text on the Windows clipboard")
	# script_reportClipboardText.category=SCRCAT_SYSTEM
	
	__gestures={
		# "kb:control+shift+!": "ceTest",  		
		"kb:control+shift+!": "listeElemFenetre",  		
		"kb:nvda+$":"listeElemFenetre",
		"kb:insert+$":"listeElemFenetre",
		"kb:alt+$":"listeElemRuban",	
		"kb:alt+shift+$":"ditOngletsRuban",		
		"kb:control+$": "applisBarreTaches", 
		"kb:control+shift+$": "applisBarreTaches2",  					
		# "kb:nvda+c": "reportClipboardText",  
		"kb:nvda+control+shift+$": "clicCentreEcran", 
		"kb:nvda+shift+$": "listeCommandesExtension", 
		u"kb:control+alt+shift+$": "searchUpdate",
		"kb:nvda+control+$":"changeMode", 
		"kb:nvda+control+pageUp":"pageUp",
		"kb:nvda+control+pageDown":"pageDown",
		"kb:control+shift+nvda+home":"firstObject", 
		"kb:nvda+control+shift+!":"statusBar",  #(desktop)
		# "kb(laptop):nvda+shift+end":"statusBar",
	}
