﻿# -*- coding: utf-8 -*-
#
# winExplore v.5.7.1 du 15.10.2024.
try:
	from urllib import urlopen
except Exception:
	from urllib.request import urlopen
import api, gui,config
import os, wx, ui
from gui import messageBox
from tones import  beep
import addonHandler
import time, winUser
import pickle

class availableUpdateDialog(wx.Dialog):
	def __init__(self, parent, title,msg): 
		super(availableUpdateDialog, self).__init__(parent, -1, title=title)
		self.parent = parent  
		titre=u"Mise à jour de l'extension WinExplore"
		mainSizer = wx.BoxSizer(wx.VERTICAL)
		sHelper = gui.guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)
		textStatic = sHelper.addItem(wx.StaticText(self))
		textStatic.SetLabel(msg)
		bHelper = gui.guiHelper.ButtonHelper(wx.HORIZONTAL)	
		infoButton = bHelper.addButton(self, wx.ID_ANY, label=u"&Aide et infos")  # wx.ID_ANY,
		yesButton = bHelper.addButton(self, wx.ID_YES, label="&Oui")
		laterButton = bHelper.addButton(self, wx.NewId(), label="&Plus tard") 
		noButton = bHelper.addButton(self, wx.ID_NO, label="&Non")
		cancelButton = bHelper.addButton(self, wx.ID_CANCEL) 
		#events
		infoButton .Bind(wx.EVT_BUTTON, self.onInfosButton) 
		yesButton.Bind(wx.EVT_BUTTON, self.onYesButton) 
		laterButton.Bind(wx.EVT_BUTTON, self.onLaterButton)
		noButton.Bind(wx.EVT_BUTTON, self.onNoButton) 
		sHelper.addDialogDismissButtons(bHelper)
		mainSizer.Add(sHelper.sizer, border=gui.guiHelper.BORDER_FOR_DIALOGS, flag=wx.ALL)
		mainSizer.Fit(self)
		self.SetSizer(mainSizer)
		self.SetTitle(titre)
		yesButton.SetFocus()
		self.CentreOnScreen()	
		option = config.conf["presentation"]["reportObjectDescriptions"]
		config.conf["presentation"]["reportObjectDescriptions"] = True
		self.Show()
		winUser.setForegroundWindow(self.GetHandle())
		api.processPendingEvents()
		config.conf["presentation"]["reportObjectDescriptions"] = option 
		
	def onInfosButton(self, event):
		urlHelp="https://sensibiliser.avh.asso.fr/fiches-informatique-adaptee/fiches_nvda/nvda/nvda_maj_winexplore.htm"
		try :
			urlopen(urlHelp)
			os.startfile (urlHelp)
		except IOError:
			msg =u"La page d'aide n'a pas été trouvée. Sélectionnez et activez un autre bouton."
			return	ui.message(msg)		

	def onYesButton(self, event):
		self.EndModal(wx.ID_YES)		
	def onLaterButton(self, event):
		self.EndModal(6666)	
	def onNoButton(self, event):
		self.EndModal(wx.ID_NO)

#variables		
version="1"
urlFileInfos="https://sensibiliser.avh.asso.fr/fiches-informatique-adaptee/fiches_nvda/data/_winexplore_actu.txt"  # version 5.7.1	
nextUpdateFile = api.config.getUserDefaultConfigPath()+"\\addons\\winExplore\\winExplore-nextUpdate.pickle"		
isDlg = False 

def checkUpdate(majAuto,twice) : 
	global isDlg
	if isDlg : return
	name, curVer="winExplore" , getCurVersion() 
	if majAuto:
		if not hasToUpdate(name) : 
			# ui.message ("pas de mise à jour")
			return
	lastVer =  getLastVersion()	
	# curVer ="6.0"  	# lastVer ="7.0"
	if curVer >= lastVer and twice==False: 
		if not majAuto:
			ui.message(name + " version " + str(curVer) + u" est à jour.")
		return 
	isDlg = True
	msg =  u"Voulez-vous mettre  à jour la version {0}  vers la version {1} ?".format(curVer, lastVer)
	with availableUpdateDialog(gui.mainFrame,u"Mise à jour de l'extension WinExplore", msg) as dlg:  
		result=dlg.ShowModal()
	if result ==6666  : wx.CallLater(40,setNextUpdate,"winExplorer",False) #later
	elif result ==wx.ID_NO  : wx.CallLater(40,setNextUpdate,"winExplorer", True) #disable
	elif result ==wx.ID_YES  : 
		doUpdate()		
	isDlg = False
	
def setNextUpdate(addonName, disabled) :  #=False
	from speech import cancelSpeech
	cancelSpeech()
	msg=""
	if not disabled :
		ut = str(time.time() + 3600 * 72) 
		# ut =str(time.time() +120)  # 2 minutes
		msg = u"La mise à jour sera reproposée dans trois jours." 
		try :
			with open(nextUpdateFile, "wb") as f:	
				pickle.dump(ut,f)  #, protocol=0
		except Exception:
			print (u"Erreur d'enregistrement de winExplore-nextUpdate.pinkle")
			pass
	else : 
		ut=str(0)
		msg= u"La mise à jour automatique a été désactivée." 
		try :
			with open(nextUpdateFile, "wb") as f:
				pickle.dump(ut,f)
		except Exception:
			print (u"Erreur d'enregistrement de winExplore-nextUpdate.pinkle 0")
			pass
	wx.CallAfter(ui.message, msg)
	
def hasToUpdate(addonName) :
	# nextUpdateFile = api.config.getUserDefaultConfigPath()+"\\addons\\winExplore-nextUpdate.pickle"	
	if  not os.path.exists(nextUpdateFile) : return True 
	now = float(time.time() )
	try :
		with open(nextUpdateFile, mode="rb") as file_obj:
			 ut=(pickle.load(file_obj))
		ut = float(ut)
		if str(ut)=="0.0" : return False # mise à jour désactivée
	except :
		ut = now - 3600
		pass
	if ut < now :
		return True
	return False		
	
def doUpdate() :
	beep(337, 1)   
	vers= getLastVersion()
	urlFile =   "https://sensibiliser.avh.asso.fr/fiches-informatique-adaptee/fiches_nvda/data/winExplore_v" + vers + ".nvda-addon"
	try :
		with urlopen  (urlFile) as data :
			data = data.read()
	except :
		# print (u"Erreur de téléchargement du fichier de la nouvelle version.")  
		messageBox(u"Le fichier de mise à jour n'a pas été trouvé.", "Erreur", wx.CLOSE|wx.ICON_WARNING)
		return  False 
	if not data or (data and len(data) < 1000): 
		# print ( u"Le fichier téléchargé est vide.")
		messageBox(u"Le fichier de mise à jour de l'extension est absent ou de format incorrect ", "Erreur", wx.CLOSE|wx.ICON_WARNING)
		return  False 
	# sauvegarde dans fichier
	beep(337, 1)
	filePath = os.getenv('temp') + "\\winExplore-update.nvda-addon"
	try :
		with open(filePath, mode="wb") as file_obj:
			file_obj.write(data)
	except : 	
		print ("Erreur d'enregistrement de:\n" + filePath )  
		messageBox(u"Erreur d'enregistrement du fichier temporaire", "Erreur", wx.CLOSE|wx.ICON_WARNING)
		return  False 		
	# lance l'installation de l'extension
	os.startfile (filePath)		
	return  True  	
		
def getCurVersion() :
	_curAddon = addonHandler.getCodeAddon()
	ver = _curAddon.manifest["version"]
	return  ver.split(" ")[0]  	

def getLastVersion() :
	with urlopen  (urlFileInfos) as data : 
		data = data.read().decode()
	lines = data.split("\n")
	v=lines[0]
	return v  		
